
#include <drivers/spi.h>
#include <drivers/flash.h>

#define FLASH_DEVICE_NAME       "SPI_2"

void spi_test_flash_api(void)
{
#define FLASH_NAME "W25Q80DV"
#define FLASH_TEST_REGION_OFFSET (0xff000)
#define FLASH_SECTOR_SIZE (4096)
#define FLASH_DEVICE DT_LABEL(DT_INST(0, jedec_spi_nor))

        const uint8_t expected[] = {0x55, 0xaa, 0x66, 0x99};
        const size_t len = sizeof(expected);
        uint8_t buf[sizeof(expected)];
        const struct device *flash_dev;
        int rc;
        
        printk("\n" FLASH_NAME " SPI flash testing\n");

        printk("==========================\n");

        flash_dev = device_get_binding(FLASH_NAME);
        if (!flash_dev)
        {
                printk("SPI flash driver %s was not found!\n",
                       FLASH_NAME);
                return;
        }

#ifdef CONFIG_FLASH_PAGE_LAYOUT
        struct flash_pages_info pages_info;
        size_t page_count, chip_size;
        page_count = flash_get_page_count(flash_dev);
        (void)flash_get_page_info_by_idx(flash_dev, 0, &pages_info);
        chip_size = page_count * pages_info.size;
        printk("Using %s, chip size: %u bytes (page: %u)\n",
               FLASH_NAME, chip_size, pages_info.size);

#endif

        /* Write protection needs to be disabled before each write or
         * erase, since the flash component turns on write protection
         * automatically after completion of write and erase
         * operations.
         */
        printk("\nTest 1: Flash erase\n");

//        rc = flash_erase(flash_dev, FLASH_TEST_REGION_OFFSET,
//                         FLASH_SECTOR_SIZE);
//        rc = flash_erase(flash_dev, 0, 1048576);
        rc = flash_erase(flash_dev, 0, 466944);

        if (rc != 0)
        {
                printk("Flash erase failed! %d\n", rc);
        }
        else
        {
                printk("Flash erase succeeded!\n");
        }

        return;
        printk("\nTest 2: Flash write\n");
        printk("Attempting to write %zu bytes\n", len);
        rc = flash_write(flash_dev, FLASH_TEST_REGION_OFFSET, expected, len);
        if (rc != 0)
        {
                printk("Flash write failed! %d\n", rc);
                return;
        }

        memset(buf, 0, len);
        rc = flash_read(flash_dev, FLASH_TEST_REGION_OFFSET, buf, len);
        if (rc != 0)
        {
                printk("Flash read failed! %d\n", rc);
                return;
        }

        if (memcmp(expected, buf, len) == 0)
        {
                printk("Data read matches data written. Good!!\n");
        }
        else
        {
                const uint8_t *wp = expected;
                const uint8_t *rp = buf;
                const uint8_t *rpe = rp + len;

                printk("Data read does not match data written!!\n");
                while (rp < rpe)
                {
                        printk("%08x wrote %02x read %02x %s\n",
                               (uint32_t)(FLASH_TEST_REGION_OFFSET + (rp - buf)),
                               *wp, *rp, (*rp == *wp) ? "match" : "MISMATCH");
                        ++rp;
                        ++wp;
                }
        }
}
